package no.ntnu.vegare.visualizer.client.auth;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.net.MalformedURLException;
import java.security.cert.X509Certificate;

import javax.net.ssl.SSLException;

import no.ntnu.vegare.visualizer.client.config.Configuration;
import no.ntnu.vegare.visualizer.client.connection.ConnectionHelper;
import no.ntnu.vegare.visualizer.client.connection.ConnectionType;
import no.ntnu.vegare.visualizer.client.connection.HttpsConnection;
import no.ntnu.vegare.visualizer.client.connection.HttpsRequest;
import no.ntnu.vegare.visualizer.client.connection.HttpsResponse;

/**
 * <p>Controls the flow of the authentication wizard.</p>
 * 
 * <p>Displays an UrlFrame to connect to the server, then an AuthFrame to 
 * prompt for a username and password.</p>
 * 
 * @see AuthFrame
 * @see UrlFrame
 * @author vegare
 * 
 */
public class AuthController implements ActionListener {

	private HttpsConnection model;

	private UrlFrame urlFrame;
	private AuthFrame authFrame;

	public AuthController(HttpsConnection model) {
		this.model = model;
	}

	/**
	 * Display UrlFrame and AuthFrame in succession, waiting for complete
	 * actions from one before moving on to the other.
	 */
	public synchronized void establishConnection() {

		urlFrame = new UrlFrame(model);
		urlFrame.addActionListener(this);
		urlFrame.setVisible(true);

		blokingWait();

		urlFrame.dispose();
		
		if (Configuration.getInstance().isUnverifiedHttpsAllowed())
			ConnectionHelper.getInstance().disableCertificateChainVerification();

		authFrame = new AuthFrame(model);
		authFrame.addActionListener(this);
		authFrame.setVisible(true);

		blokingWait();

		authFrame.dispose();
	}

	/**
	 * Wait for the JFrame thread to close. Notifications are thrown by
	 * connect() and login() as response to actions performed in the frames.
	 */
	private synchronized void blokingWait() {
		try {
			wait();
		} catch (InterruptedException e) {
			Thread.currentThread().interrupt();
		}
	}

	/**
	 * Connect to the server at the given URL, and request a X509 certificate.
	 */
	private void connect() {

		String url = urlFrame.getUrl();

		// Validate URL
		try {
			model.setUrl(url);
		} catch (MalformedURLException e) {
			urlFrame.displayError("Invalid URL");
			return;
		}

		// Check for unencrypted connection
		if (!Configuration.getInstance().isHttpAllowed()
				&& model.getConnectionType() == ConnectionType.HTTP_CONNECTION) {
			
			urlFrame.displayError("HTTP is not allowed.");
			return;
		}

		// Fetch certificate
		try {
			X509Certificate cert = ConnectionHelper.getInstance().requestCertificate(model.getUrl());
			model.setCertificate(cert);
		} catch (SSLException e) {
			model.setConnectionType(ConnectionType.HTTPS_UNTRUSTED);		
		} catch (IOException e) {
			urlFrame.displayError("Could not connect to server");
			return;
		}
		
		// Check for unverified connection
		if (!Configuration.getInstance().isUnverifiedHttpsAllowed()
				&& model.getConnectionType() == ConnectionType.HTTPS_UNTRUSTED) {
			
			urlFrame.displayError("Server is not in trusted path");
			return;
		}
		
		notify();
	}

	/**
	 * <p>Connect to the main page without any parameters, simply to test the
	 * authentication parameters.</p>
	 * 
	 * <p>Uses HTTP Basic Authentication to be compatible with browsers.</p>
	 * 
	 * <ul>
	 *  <li>If HTTP code 401 is received, authentication failed.</li>
	 *  <li>If HTTP code 200 is received, authentication was successful.</li>
	 * </ul>
	 */
	private void login() {

		model.setUsername(authFrame.getUsername());
		model.setPassword(authFrame.getPassword());

		HttpsRequest req = new HttpsRequest(model.getUrl());
		req.setAuthorization("Basic", model.getUsername(), model.getPassword());
				
		HttpsResponse res;
		try {
			res = ConnectionHelper.getInstance().request(req);
		} catch (IOException e) {
			authFrame.displayError("Could not connect to server");
			e.printStackTrace();
			return;
		}
		
		switch (res.getResponseCode()) {
			case 401:	authFrame.displayError("Authentication failed");
						break;
			case 200:	notify();
						break;
			default:	authFrame.displayError("Error " 
											 + res.getResponseCode()
											 + " from server.");
						break;
		}
	}

	@Override
	public synchronized void actionPerformed(ActionEvent e) {

		if ("connect".equals(e.getActionCommand())) {
			connect();
		} else if ("login".equals(e.getActionCommand())) {
			login();
		}
	}
}
