package no.ntnu.vegare.visualizer.client.connection;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.URL;
import java.security.KeyManagementException;
import java.security.NoSuchAlgorithmException;
import java.security.cert.Certificate;
import java.security.cert.X509Certificate;

import javax.net.ssl.HostnameVerifier;
import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLException;
import javax.net.ssl.SSLSession;
import javax.net.ssl.TrustManager;
import javax.net.ssl.X509TrustManager;

/**
 * <p>Thread safe singleton implementation by Bill Pugh:
 * <a href="http://en.wikipedia.org/wiki/Singleton_pattern">
 * http://en.wikipedia.org/wiki/Singleton_pattern</a></p>
 * 
 * <p>Provides an interface towards the HTTP or HTTPS connection established
 * in the authentication phase. Plain text access can be disabled in the
 * configuration class.</p>
 * 
 * @author vegare
 *
 */
public class ConnectionHelper {

	// Private constructor prevents instantiation from other classes
	private ConnectionHelper() {}
	
	/**
	 * SingletonHolder is loaded on the first execution of Singleton.getInstance() 
	 * or the first access to SingletonHolder.INSTANCE, not before.
	 */
	private static class SingletonHolder { 
		public static final ConnectionHelper INSTANCE = new ConnectionHelper();
	}
	 
	public static ConnectionHelper getInstance() {
		return SingletonHolder.INSTANCE;
	}

	/**
	 * <p>Request the certificate from a HTTPS server. In this prototype the
	 * certificate is not validated, but simply shown to the user as a
	 * basic identification effort.</p>
	 * 
	 * <p>This is <strong>not</strong> meant as a security measure, but as a
	 * usability enhancement. In a production system, proper security should
	 * be implemented.</p>
	 * 
	 * <p>Because no Authorization request property is added, the results will
	 * be a 401 error page, which is immediately discarded.</p>
	 * 
	 * @throws IOException	Malformed URL (should be impossible)
	 * 						or connection timeout
	 * 
	 * @throws SSLException	Unverified certificate trust path
	 */
	public synchronized X509Certificate requestCertificate(String url) throws IOException,
																			  SSLException {
				
		URL validUrl = new URL(url);
		
		if (!validUrl.getProtocol().equals("https"))
			return null;
				
		X509Certificate cert = null;
		
		HttpsURLConnection con = (HttpsURLConnection)validUrl.openConnection();
		con.setConnectTimeout(3000);
		con.connect();
		
		Certificate[] certs = con.getServerCertificates();
		cert = (X509Certificate) certs[0];
		
		con.disconnect();

		return cert;
	}

	/**
	 * <p>Make a HTTPS request, receive a response. Currently keep-alive is
	 * not implemented, thus each request is a separate connection.</p>
	 * 
	 * @throws IOException	Malformed URL, connection timeout or unverified
	 * 						certificate, if validation is not disabled.
	 */
	public synchronized HttpsResponse request(HttpsRequest req) throws IOException {
				
		HttpURLConnection con;
		
		URL url = new URL(req.getUrl());
		
		if (url.getProtocol().equals("https"))
			con = (HttpsURLConnection)url.openConnection();
		else
			con = (HttpURLConnection)url.openConnection();
		
		con.setRequestProperty("Authorization", req.getAuthorization());
		con.setConnectTimeout(3000);
		con.connect();
		
		HttpsResponse res = new HttpsResponse();
		
		res.setResponseCode(con.getResponseCode());
		res.setResponseMessage(con.getResponseMessage());
		
		if (res.getResponseCode() == 200)
			res.setContent(readContent(con));
	    		
		con.disconnect();
		
		return res;
	}
	
	/**
	 * <p>Read the message content of a HTTPS request.</p>
	 */
	private synchronized String readContent(HttpURLConnection con) throws IOException {
		
		BufferedReader reader = new BufferedReader(new InputStreamReader(con.getInputStream()));
		StringBuilder stringBuilder = new StringBuilder();

		String line = null;
		while ((line = reader.readLine()) != null)
		{
		   stringBuilder.append(line + "\n");
		}

		return stringBuilder.toString();
	}
	
	/**
	 * <p>This application is likely to be used in a local environment, where
	 * certificates are self-signed. Therefore, this class provides a measure
	 * to disable chain verification.</p>
	 * 
	 * <p>Should not be used in a production setting.</p>
	 */
	public void disableCertificateChainVerification() {
		
		// Create a trust manager that does not validate certificate chains
		TrustManager[] trustAllCerts = new TrustManager[] {new X509TrustManager() {
				public java.security.cert.X509Certificate[] getAcceptedIssuers() {
					return null;
				}
				public void checkClientTrusted(X509Certificate[] certs, String authType) {
				}
				public void checkServerTrusted(X509Certificate[] certs, String authType) {
				}
			}
		};

		// Install the all-trusting trust manager
		SSLContext sc;
		try {
			sc = SSLContext.getInstance("SSL");
			sc.init(null, trustAllCerts, new java.security.SecureRandom());
			HttpsURLConnection.setDefaultSSLSocketFactory(sc.getSocketFactory());
		} catch (NoSuchAlgorithmException e) {
			e.printStackTrace();
		} catch (KeyManagementException e) {
			e.printStackTrace();
		}

		// Create all-trusting host name verifier
		HostnameVerifier allHostsValid = new HostnameVerifier() {
			public boolean verify(String hostname, SSLSession session) {
				return true;
			}
		};

		// Install the all-trusting host verifier
		HttpsURLConnection.setDefaultHostnameVerifier(allHostsValid);
	}
}