package no.ntnu.vegare.visualizer.client.connection;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;

import javax.swing.JOptionPane;

import com.google.gson.Gson;
import com.google.gson.JsonParseException;

import no.ntnu.vegare.visualizer.client.config.Configuration;
import no.ntnu.vegare.visualizer.client.db.AbstractLogDatabase;
import no.ntnu.vegare.visualizer.client.db.LogEntry;
import no.ntnu.vegare.visualizer.client.utils.TimeUtils;

/**
 * <p>Runs in the background as a separate thread, constantly requesting new
 * log entries from the server. Feeds the database directly.</p>
 * 
 * <p>Uses automatic JSON parsing to generate <code>LogEntry</code> objects
 * from the incoming text.</p>
 * 
 * @author vegare
 *
 */
public class ServerPollDaemon implements Runnable {

	private HttpsConnection conn;
	private AbstractLogDatabase db;
	
	private Gson gson;
	
	private int lastTimestamp;
	private int currTimestamp;
	
	/**
	 * Launch a new server poll daemon.
	 * 
	 * @param conn HTTP connection instance
	 * @param db Log database to feed with incidents
	 */
	public ServerPollDaemon(HttpsConnection conn, AbstractLogDatabase db) {
		this.conn = conn;
		this.db = db;
		
		gson = new Gson();
		
		// Kickstart with a backlog of log data
		currTimestamp = TimeUtils.getCurrentTime();
		lastTimestamp = currTimestamp - Configuration.getInstance().getBacklogSize();
	}
	
	@Override
	public void run() {
		
		while(true) {
			
			currTimestamp = TimeUtils.getCurrentTime();

			HttpsRequest request = new HttpsRequest(conn.getUrl());
			request.setAuthorization("Basic", conn.getUsername(), conn.getPassword());
			request.addParameter("action", "update");
			request.addParameter("timeFrom", Integer.toString(lastTimestamp));
			request.addParameter("timeTo", Integer.toString(currTimestamp));
			
			try {
				requestUpdates(request);
				
			} catch (IOException e) {
				
				// A cheap way out. If time, make an UnhandledExceptionHandler,
				// which tries to establish a new server poll daemon. For now,
				// the daemon simply dies, and the user is left with the 
				// entries in the database at the moment of the crash.
				JOptionPane.showMessageDialog(null,
						"The connection to the server was lost. Please restart " +
					    "the application to establish a new connection",
						"Connection lost",
						JOptionPane.INFORMATION_MESSAGE);
				
				break;
			}
			
			lastTimestamp = currTimestamp;
		
			try {
				Thread.sleep(Configuration.getInstance().getSleepTime());
			} catch (InterruptedException e) {
				Thread.currentThread().interrupt();
			}
		}
	}

	/**
	 * Request new updates. If any are received, add to db
	 */
	private void requestUpdates(HttpsRequest request) throws IOException {
		
		HttpsResponse response = ConnectionHelper.getInstance().request(request);
		
		if (response.getResponseCode() == 200)
		{
			Collection<LogEntry> entries = new ArrayList<LogEntry>(20);
			
			String[] update = response.getContent().split("\n");
			for (String line : update) {
				parseLine(entries, line);
			}
			db.addEntries(entries);
		}
	}

	/**
	 * Parse a single JSON log line, add to collection if valid
	 */
	private void parseLine(Collection<LogEntry> entries, String line) {
		
		try {
			LogEntry e = gson.fromJson(line, LogEntry.class);
			
			if (e != null)
				entries.add(e);
		} catch (JsonParseException e) {
			e.printStackTrace();
		}
	}
}
