package no.ntnu.vegare.visualizer.client.db;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;

import javax.swing.table.AbstractTableModel;

import no.ntnu.vegare.visualizer.client.utils.TimeUtils;

/**
 * <p>Collection of log entries. Works as a table model, feeding the overview
 * screen and the filter view automatically, if used correctly.</p>
 * 
 * <p>You can use either <code>setQueryFilter()</code> to modify content
 * according to rules, or <code>setEntries()</code> to set a specific set.</p>
 * 
 * @author vegare
 *
 */
public class LogEntrySet extends AbstractTableModel implements DatabaseObserver {

	private static final long serialVersionUID = -2830987912856189680L;

	public final Object logEntryLock = new Object();
	
	private String[] severity = new String[]{"Emergency","Alert",
											 "Critical", "Error",
											 "Warning", "Notice",
											 "Debug", "Info"};
	
	private String[] columnNames = new String[] { "ID", "Facility", "Priority",
			"IP Address", "City", "Country", "Time" };

	private AbstractLogDatabase db;
	private QueryFilter filter;

	// List of current entries. Must be synchronized,
	// as it's accessed by the server poll daemon.
	private List<LogEntry> logEntries;

	// Whether to update entries "live"
	// from the database
	private boolean live;

	// The timestamp this model was last valid.
	// Only used in the overview panel.
	private int currentTime;

	/**
	 * Create a new log entry collection, based on a database instance
	 * 
	 * @param db Database instance
	 */
	public LogEntrySet(AbstractLogDatabase db) {

		this.db = db;

		filter = new QueryFilter();
		logEntries = Collections.synchronizedList(new ArrayList<LogEntry>(0));
		live = true;

		db.addDatabaseObserver(this);
	}

	/**
	 * @param row Row number in current log entry set
	 * @return Corresponding row
	 */
	public LogEntry getEntry(int row) {
		return logEntries.get(row);
	}

	/**
	 * <p>When a new query filter is set, the entire model is invalidated and
	 * fetched from the database. Used when the user specified a filter, or
	 * when the application only needs a subset.</p>
	 */
	public void setQueryFilter(QueryFilter filter) {
		
		this.filter = filter;

		Collection<LogEntry> entries = db.query(filter);
		
		synchronized(logEntryLock) {
			logEntries.clear();
			logEntries.addAll(entries);
		}
		
		live = true;
		
		fireTableDataChanged();
	}
	
	/**
	 * <p>When a entry set is set, the entire model is invalidated and fetched
	 * from the database. Used when the user or software wants a specific
	 * set of entries, rather than a rule based collection.</p>
	 * 
	 * @param entries
	 */
	public void setEntries(Collection<LogEntry> entries) {

		synchronized(logEntryLock) {
			logEntries.clear();
			logEntries.addAll(entries);
		}
		
		live = false;
		
		fireTableDataChanged();
	}

	/**
	 * @return Whether the entry set is receiving live updates
	 */
	public boolean isLive() {
		return live;
	}

	/**
	 * @return All log entries
	 */
	public Collection<LogEntry> getEntries() {
		return logEntries;
	}	

	@Override
	public void entriesAdded(final Collection<LogEntry> entries) {
		
		if (!live)
			return;
		
		synchronized(logEntryLock) {
			for (LogEntry e : entries) {
				if (filter.matches(e))
					logEntries.add(e);
			}
		}
		
		fireTableDataChanged();
	}

	/**
	 * <p>This is <strong>not</strong> the current system time, but rather
	 * the time the entry set is located at. If we are watching a historical
	 * overview, this method returns the time of observation.</p>
	 * 
	 * @return The current time of the entry set.
	 */
	public int getCurrentTime() {
		return currentTime;
	}

	/**
	 * <p>Set the current time watched by the user.</p>
	 * 
	 * @param currentTime Time specified by user.
	 */
	public void setCurrentTime(int currentTime) {
		this.currentTime = currentTime;
	}

	@Override
	public int getRowCount() {
		return logEntries.size();
	}

	@Override
	public int getColumnCount() {
		return columnNames.length;
	}

	@Override
	public String getColumnName(int col) {
		return columnNames[col];
	}

	@Override
	public Object getValueAt(int rowIndex, int columnIndex) {

		LogEntry entry = logEntries.get(rowIndex);

		// TODO: Add custom renderer
		switch (columnIndex) {
		case 0:
			return entry.getId();
		case 1:
			return entry.getFacility();
		case 2:
			return severity[entry.getSeverity()];
		case 3:
			return entry.getIpaddress();
		case 4:
			return (entry.getLocation() != null
				&&  entry.getLocation().city != null
				&&	!entry.getLocation().city.equals(""))
				 ?  entry.getLocation().city
				 :  "Unknown";
		case 5:
			return (entry.getLocation() != null
				&&  entry.getLocation().countryName != null
				&&	!entry.getLocation().countryName.equals(""))
				 ?  entry.getLocation().countryName
				 :  "Unknown";
		case 6:
			return TimeUtils.timestampToDate(entry.getTimestamp());
		}

		return null;
	}

	public int getMaxPriority() {
		
		int minPri = Integer.MAX_VALUE;
		
		for (LogEntry e : logEntries) {
			if (e.getSeverity() < minPri)
				minPri = e.getSeverity();
		}
		
		return minPri;
	}
}
