package no.ntnu.vegare.visualizer.client.db;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Collection;

/**
 * <p>A simplified implementation of a SQLite database interface.</p>
 * 
 * <p>Uses SqliteJDBC (<a href="http://www.zentus.com/sqlitejdbc/">
 * http://www.zentus.com/sqlitejdbc/</a>) to create an in-memory database, and
 * provides an interface to insert or load entries.</p>
 * 
 * <p>The database is cleared when the process dies.</p>
 * 
 * @author vegare
 *
 */
public class SQLiteDatabase extends AbstractLogDatabase {

	public static final Object dbLock = new Object();
	
	private Connection dbConn;
	
	/**
	 * Create an in-memory database and a table for log records.
	 */
	public SQLiteDatabase() {
		
		super();
		
		try {
			Class.forName("org.sqlite.JDBC");
		
		    dbConn = DriverManager.getConnection("jdbc:sqlite::memory:");
		    
		    Statement stat = dbConn.createStatement();
		    stat.executeUpdate("drop table if exists log;");
		    stat.executeUpdate("create table log (id, timestamp, severity, facility, ipaddress, message);");
		      
		} catch (Exception e) {
			// Oh, the horror... Just a prototype!
			e.printStackTrace();
		}
	}
	
	@Override
	public void addEntries(Collection<LogEntry> entries) {
		
		if (entries.size() == 0) return;
		
		synchronized (dbLock) {
			for (LogEntry e : entries)
			{
				try {
				    PreparedStatement prep = dbConn.prepareStatement(
				      "insert into log values (?, ?, ?, ?, ?, ?);");
			
				    prep.setInt(1, e.getId());
				    prep.setInt(2, e.getTimestamp());
				    prep.setInt(3, e.getSeverity());
				    prep.setString(4, e.getFacility());
				    prep.setString(5, e.getIpaddress());
				    prep.setString(6, e.getMessage());
				    
				    prep.addBatch();
			
				    dbConn.setAutoCommit(false);
				    prep.executeBatch();
				    dbConn.setAutoCommit(true);
				      
				} catch (Exception ex) {
					// Oh, the horror... Just a prototype!
					ex.printStackTrace();
				}
			}
		}

		fireEntriesAdded(entries);
	}
	
	/**
	 * Return a list of all incidents in the database
	 */
	public ArrayList<LogEntry> getAll() {
			
		try {
			
			ArrayList<LogEntry> entries = new ArrayList<LogEntry>(100);

			synchronized(dbLock) {
				
				PreparedStatement prep = dbConn.prepareStatement("select * from log;");
			    
			    ResultSet rs = prep.executeQuery();
			    
			    while (rs.next()) {
	
				    LogEntry entry = extractLogEntry(rs);		    	
			    	entries.add(entry);
			    }
			}
		    
		    return entries;
		      
		} catch (Exception ex) {
			// Oh, the horror... Just a prototype!
			ex.printStackTrace();
		}
		
		return null;
	}
	
	@Override
	public ArrayList<LogEntry> query(QueryFilter filter) {
				
		try {
			ArrayList<LogEntry> entries = new ArrayList<LogEntry>(100);
			
			String sql = "select * from log where " + 
						 "timestamp >= ? and " +
						 "timestamp <= ? and " +
						 "severity <= ? and " +
						 "facility LIKE '%" + filter.getFacility() + "%' and " +
						 "ipAddress LIKE '" + filter.getIpPrefix() + "%' and " + 
						 "message LIKE '%" + filter.getMessageContains() + "%';";

			synchronized(dbLock) {
				PreparedStatement prep = dbConn.prepareStatement(sql);
				
			    prep.setInt(1, filter.getMinTimestamp());
			    prep.setInt(2, filter.getMaxTimestamp());
			    prep.setInt(3, filter.getMinPriority());
			    
			    ResultSet rs = prep.executeQuery();
			    
			    while (rs.next()) {
	
				    LogEntry entry = extractLogEntry(rs);		    	
			    	entries.add(entry);
			    }	
			}
			
			return entries;
			
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}	
		return null;
	}

	private LogEntry extractLogEntry(ResultSet rs) throws SQLException {
		LogEntry entry = new LogEntry();
		entry.setId(rs.getInt("id"));
		entry.setTimestamp(rs.getInt("timestamp"));
		entry.setSeverity(rs.getInt("severity"));
		entry.setFacility(rs.getString("facility"));
		entry.setIpaddress(rs.getString("ipaddress"));
		entry.setMessage(rs.getString("message"));
		return entry;
	}
}
