package no.ntnu.vegare.visualizer.client.main;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import no.ntnu.vegare.visualizer.client.db.AbstractLogDatabase;
import no.ntnu.vegare.visualizer.client.db.LogEntry;
import no.ntnu.vegare.visualizer.client.db.LogEntrySet;
import no.ntnu.vegare.visualizer.client.main.details.DetailPanel;
import no.ntnu.vegare.visualizer.client.main.details.LogReport;
import no.ntnu.vegare.visualizer.client.main.details.LogReportSet;
import no.ntnu.vegare.visualizer.client.main.details.ReportController;
import no.ntnu.vegare.visualizer.client.main.filter.FilterController;
import no.ntnu.vegare.visualizer.client.main.filter.FilterPanel;
import no.ntnu.vegare.visualizer.client.main.overview.OverviewController;
import no.ntnu.vegare.visualizer.client.main.overview.OverviewPanel;
import no.ntnu.vegare.visualizer.client.main.overview.history.TimeSeries;
import no.ntnu.vegare.visualizer.client.main.overview.map.IncidentWheel;

/**
 * <p>Handles all interaction between the three main panels:</p>
 * 
 * <ul>
 *  <li>Click on incident wheels</li>
 *  <li>Click on gauge</li>
 *  <li>Double click on entries in filter view</li>
 * </ul>
 * 
 * @author vegare
 *
 */
public class MainController implements ActionListener {

	private AbstractLogDatabase db;
	
	private MainFrame view;

	private LogEntrySet logEntrySet;
	private TimeSeries timeSeries;
	private OverviewPanel overviewPanel;
	
	private LogEntrySet logTableModel;
	private FilterPanel filterPanel;
	
	private LogReportSet logReportSet;
	private DetailPanel detailPanel;
	
	/**
	 * Create the models and controllers for the three panels.
	 * 
	 * @param db Underlying database instance
	 */
	public MainController(AbstractLogDatabase db) {
	
		this.db = db;
		
		// Initialize overview tab
		logEntrySet = new LogEntrySet(db);		
		timeSeries = new TimeSeries(db);
		
		overviewPanel = new OverviewPanel(logEntrySet, timeSeries);
		overviewPanel.addActionListener(this);
		
		new OverviewController(db, logEntrySet, timeSeries, overviewPanel);
		
		// Initialize filter tab
		logTableModel = new LogEntrySet(db);
		
		filterPanel = new FilterPanel(logTableModel);
		filterPanel.addActionListener(this);
		
		new FilterController(db, logTableModel, filterPanel);
		
		// Initialize report tab
		logReportSet = new LogReportSet();
		detailPanel = new DetailPanel(logReportSet);
		
		new ReportController(logReportSet, detailPanel);
		
		// Initialize main window
		view = new MainFrame();
		
		view.addTab(overviewPanel);
		view.addTab(filterPanel);
		view.addTab(detailPanel);
	}

	public void launchGui() {
		
		view.setVisible(true);
	}
	
	@Override
	public void actionPerformed(ActionEvent e) {
		
		if ("incidentWheel".equals(e.getActionCommand())) {

			logTableModel.setEntries(((IncidentWheel)e.getSource()).getEntries());
			view.switchTab(filterPanel);
		
		} else if ("gauge".equals(e.getActionCommand())) {
			
			logTableModel.setEntries(logEntrySet.getEntries());
			view.switchTab(filterPanel);
				
		} else if ("filter".equals(e.getActionCommand())) {
			
			LogReport report = new LogReport(db, (LogEntry) e.getSource());
			logReportSet.addEntry(report);
			view.switchTab(detailPanel);
			
		}
	}
}
