package no.ntnu.vegare.visualizer.client.main.overview;

import java.awt.event.ActionListener;
import java.beans.PropertyChangeListener;

import javax.swing.JPanel;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;

import no.ntnu.vegare.visualizer.client.db.LogEntrySet;
import no.ntnu.vegare.visualizer.client.db.QueryFilter;
import no.ntnu.vegare.visualizer.client.main.overview.gauge.GaugePanel;
import no.ntnu.vegare.visualizer.client.main.overview.history.HistoryPanel;
import no.ntnu.vegare.visualizer.client.main.overview.history.TimeSeries;
import no.ntnu.vegare.visualizer.client.main.overview.map.MapOverlay;
import no.ntnu.vegare.visualizer.client.main.overview.map.MapPanel;
import no.ntnu.vegare.visualizer.client.utils.TimeUtils;

/**
 * <p>The view of the overview panel. Displays three components:</p>
 * 
 * <ul>
 *  <li>World map with incident wheels</li>
 *  <li>History graph</li>
 *  <li>Gauge</li>
 * </ul>
 *  
 * <p>Contains an animation thread, which updates the graphical components.</p>
 * 
 * @author vegare
 *
 */
public class OverviewPanel extends JPanel implements TableModelListener {

	private static final long serialVersionUID = -4558384390514508405L;

	private LogEntrySet entrySet;
	private TimeSeries timeSeries;
	
	private MapOverlay mapOverlay;
	private MapPanel mapPanel;
	private HistoryPanel historyPanel;
	private GaugePanel gaugePanel;
	
	/**
	 * Create a new overview panel.
	 * 
	 * @param entrySet Model for map and gauge
	 * @param timeSeries Model for history graph
	 */
	public OverviewPanel(LogEntrySet entrySet, TimeSeries timeSeries) {
		
		this.entrySet = entrySet;
		this.timeSeries = timeSeries;
		
		setName("Overview");
		
		initializeComponents();
		
		setupAnimation();
		
		entrySet.addTableModelListener(this);
	}
	
	private void initializeComponents() {
		
		setLayout(null);
		
		mapOverlay = new MapOverlay();
		mapOverlay.setLocation(0, 0);
		mapOverlay.setSize(1000, 21);
		
		mapPanel = new MapPanel(entrySet);
		mapPanel.setLocation(0, 21);
		mapPanel.setSize(1000, 479);
		
		historyPanel = new HistoryPanel(entrySet, timeSeries);
		historyPanel.setLocation(0, 500);
		historyPanel.setSize(800, 190);
		
		gaugePanel = new GaugePanel(entrySet);
		gaugePanel.setLocation(800, 500);
		gaugePanel.setSize(200, 190);
		
		add(mapOverlay);
		add(mapPanel);
		add(historyPanel);
		add(gaugePanel);
	}

	/**
	 * <p>Launch a new animation thread, which makes sure the world map and
	 * history graph is always up to date. FPS is only 1 because no smaller
	 * time interval results in changes on the screen.</p>
	 */
	private void setupAnimation() {
		
        new Thread(
                new Runnable() {
                    public void run() {
                    	while (true) {
                    		
	                        if (entrySet.isLive()) {
	                        	updateFilter();
	                        }
	                        repaint();
	                        
	                        try {
	                            Thread.sleep(1000);
	                        } catch (Exception e) {
	                            e.printStackTrace();
	                        }
                    	}
                    }
                }).start();
	}
	
	/**
	 * <p>Set the query filter for the world map. Always current time minus
	 * one hour, which is the life time of incident wheels.</p>
	 */
	private void updateFilter() {
		
        QueryFilter filter = new QueryFilter();
        filter.setMinTimestamp(TimeUtils.getCurrentTime() - TimeUtils.HOUR);
        entrySet.setQueryFilter(filter);
	}
	
	public void addActionListener(ActionListener listener) {
		mapPanel.setActionListener(listener);
		gaugePanel.addActionListener(listener);
	}

	@Override
	public void addPropertyChangeListener(PropertyChangeListener listener) {
		super.addPropertyChangeListener(listener);
		historyPanel.addPropertyChangeListener(listener);
	}

	@Override
	public void tableChanged(TableModelEvent e) {
		mapPanel.updateWheels();
	}
}
