package no.ntnu.vegare.visualizer.client.main.overview.history;

import java.awt.Color;
import java.awt.Cursor;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.image.BufferedImage;
import java.text.SimpleDateFormat;
import java.util.Date;

import javax.swing.JPanel;

import no.ntnu.vegare.visualizer.client.db.LogEntrySet;
import no.ntnu.vegare.visualizer.client.utils.TimeUtils;

/**
 * <p>Holds the panel with the history graph. Uses an underlying
 * <code>TimeSeries</code> to draw the graph with minimal overhead.</p>
 * 
 * @see TimeSeries
 * @author vegare
 *
 */
public class HistoryPanel extends JPanel implements MouseListener, MouseMotionListener {

	private static final long serialVersionUID = -6911249553296044029L;

	private LogEntrySet logEntrySet;
	private TimeSeries timeSeries;
	
	private Cursor blankCursor;
	
	private boolean manualMarker;
	private int currentMarkerX;
	
	private int minX;
	private int maxX;
	private int minY;
	private int maxY;
	
	private int minTime;
	private int maxTime;

	public HistoryPanel(LogEntrySet logEntrySet, TimeSeries timeSeries) {
		
		this.logEntrySet = logEntrySet;
		this.timeSeries = timeSeries;
		
		BufferedImage cursorImg = new BufferedImage(16, 16, BufferedImage.TYPE_INT_ARGB);
		blankCursor = Toolkit.getDefaultToolkit().createCustomCursor(
			    cursorImg, new Point(0, 0), "blank cursor");
		
		manualMarker = false;
		currentMarkerX = 0; // Changed before drawn anyway
		
		minX = 2;
		maxX = 785;
		minY = 30;
		maxY = 160;
		
		updateTime();
		
		addMouseListener(this);
		addMouseMotionListener(this);
	}

	private void updateTime() {
		maxTime = TimeUtils.round(TimeUtils.getCurrentTime(), TimeUtils.HOUR);
		minTime = maxTime - 72*TimeUtils.HOUR;
	}
	
	@Override
	protected void paintComponent(Graphics g) {
		super.paintComponent(g);

		updateTime();
		
		g.setColor(Color.BLACK);
		g.fillRect(0, 0, getWidth()-2, 21);
		
		g.setColor(Color.DARK_GRAY);
		g.fillRect(2, 2, getWidth()-4, 17);
		
		g.setColor(Color.WHITE);
		g.drawString("History", 5, 14);
		
		g.setColor(Color.BLACK);
		g.fillRect(0, 22, getWidth()-2, getHeight()-26);
		
		drawGrid(g);
		drawGraph(g);
		drawMarker(g);
	}

	private void drawGrid(Graphics g) {
		
		g.setColor(Color.WHITE);

		int firstTick = maxTime - (maxTime % (24*60*60)) - 2*60*60;
		
		// Big ticks
		int ticks[] = new int[]{
				firstTick,
				firstTick - 1*24*60*60,
				firstTick - 2*24*60*60,
				firstTick - 3*24*60*60,
			};
		
		for (int i : ticks) {
			
			String txt = new SimpleDateFormat("d MMM").format(new Date(1000L*i));
						
			int pos = timeToPos(i) - (g.getFontMetrics().stringWidth(txt)/2);
			g.drawString(txt, pos, 180);
		}
			
		// Small ticks
		for (int i=-10; i<4; i++) {
			if ((i % 4) != 0)
				g.fillOval(timeToPos(firstTick + i*6*60*60), 174, 3, 3);
		}	
	}

	private void drawGraph(Graphics g) {
		
		g.setColor(Color.GREEN);
		
		int curTime = timeSeries.getSize();
		int intervalSize = TimeSeries.bucketSize;
		int endTime = curTime - 72*TimeUtils.HOUR;
		
		int vMin = timeSeries.getMinValue();
		int vMax = timeSeries.getMaxValue();

		while (curTime > endTime) {
			
			int x = timeToPos(curTime);
			int y = valueToY(timeSeries.get(curTime), vMin, vMax);
			
			g.drawLine(x, maxY, x, y);
			
			curTime -= intervalSize;
		}
	}

	private void drawMarker(Graphics g) {

		if (!manualMarker && logEntrySet.isLive()) {
			g.setColor(Color.BLUE);
			g.fillRect(785, 22, 10, maxY-21);
		} else if (!manualMarker && !logEntrySet.isLive()) {
			g.setColor(Color.RED);
			g.fillRect(timeToPos(logEntrySet.getCurrentTime())-10, 22, 10, maxY-21);
		} else if (currentMarkerX > getWidth()-10) {
			g.setColor(Color.BLUE);
			g.fillRect(currentMarkerX-10, 22, 10, maxY-21);
		} else {
			g.setColor(Color.YELLOW);
			g.fillRect(currentMarkerX-10, 22, 10, maxY-21);
		}
	}
	
	private int valueToY(int val, int vMin, int vMax) {
		
		double progression = ((double)val-vMin)/(vMax-vMin);
				
		return maxY - (int) (progression*(maxY-minY));
	}
	
	private int posToTime(int pos) {
		
		double progression = ((double)pos-minX)/(maxX-minX);
				
		return (int) (progression*(maxTime-minTime) + minTime);
	}
	
	private int timeToPos(int time) {
		
		double progression = ((double)time-minTime)/(maxTime-minTime);
		
		return (int) (progression*(maxX-minX) + minX);
	}

	@Override
	public void mouseEntered(MouseEvent e) {
		currentMarkerX = e.getX();
		manualMarker = true;
		setCursor(blankCursor);
		repaint();
	}

	@Override
	public void mouseExited(MouseEvent e) {
		manualMarker = false;
		setCursor(Cursor.getDefaultCursor());
		repaint();
	}

	@Override
	public void mouseMoved(MouseEvent e) {
		currentMarkerX = e.getX();
		repaint();
	}

	@Override
	public void mouseClicked(MouseEvent e) {
		
		if (e.getX() > maxX)
			firePropertyChange("live", false, true);
		else
			firePropertyChange("time", logEntrySet.getCurrentTime(), posToTime(e.getX()));
	}
	
	@Override
	public void mouseDragged(MouseEvent e) {}

	@Override
	public void mousePressed(MouseEvent e) {}

	@Override
	public void mouseReleased(MouseEvent e) {}
}
