package no.ntnu.vegare.visualizer.client.main.overview.history;

import java.util.Collection;
import java.util.Collections;
import java.util.SortedMap;
import java.util.TreeMap;

import javax.swing.AbstractListModel;

import no.ntnu.vegare.visualizer.client.db.AbstractLogDatabase;
import no.ntnu.vegare.visualizer.client.db.DatabaseObserver;
import no.ntnu.vegare.visualizer.client.db.LogEntry;
import no.ntnu.vegare.visualizer.client.utils.TimeUtils;

/**
 * <p>A model representing the history graph. Holds a list of intervals with
 * corresponding incident counts.</p>
 * 
 * @author vegare
 *
 */
public class TimeSeries extends AbstractListModel implements DatabaseObserver {

	private static final long serialVersionUID = -2137624698427125246L;

	// Map of current series. Must be synchronized,
	// as it's accessed by the server poll daemon.
	private SortedMap<Integer, Integer> series;	
	
	public static final int bucketSize = 15*TimeUtils.MINUTE;
	
	// Vertical min/max
	private int minVal;
	private int maxVal;

	public TimeSeries(AbstractLogDatabase db) { 
		
		series = Collections.synchronizedSortedMap(new TreeMap<Integer, Integer>());
		
		entriesAdded(db.getAll());
		db.addDatabaseObserver(this);
	}

	@Override
	public void entriesAdded(final Collection<LogEntry> entries) {
		
		synchronized(series) {
			for (LogEntry e : entries) {
				
				int key = TimeUtils.round(e.getTimestamp(), bucketSize);
				
				if (series.containsKey(key)) {
					int oldVal = series.get(key);
					series.put(key, oldVal+1);
				} else {
					series.put(key, 1);
				}
			}
		}
		
		minVal = Integer.MAX_VALUE;
		maxVal = Integer.MIN_VALUE;
		
		for (Integer key : series.keySet()) {
			if (key == series.lastKey()) continue;
			
			int val = series.get(key);
			
			if (val > maxVal) maxVal = val;
			if (val < minVal) minVal = val;
		}
		
		// Fire event to update view
		fireContentsChanged(this, 0, getSize());
	}

	public int getMinValue() {
		return minVal;
	}

	public int getMaxValue() {
		return maxVal;
	}

	public int get(int index) {
		return series.containsKey(index) ? series.get(index) : 0;
	}
	
	@Override
	public int getSize() {
		return series.lastKey();
	}

	@Override
	public Object getElementAt(int index) {
		return series.containsKey(index) ? series.get(index) : 0;
	}
}