package no.ntnu.vegare.visualizer.client.main.overview.map;

import java.awt.Color;
import java.awt.Cursor;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.ArrayList;
import java.util.Collection;

import javax.swing.JButton;

import no.ntnu.vegare.visualizer.client.db.LogEntry;
import no.ntnu.vegare.visualizer.client.db.LogEntrySet;
import no.ntnu.vegare.visualizer.client.utils.Resources;
import no.ntnu.vegare.visualizer.client.utils.TimeUtils;

/**
 * <p>The visual representation of a single incident wheel.</p>
 * 
 * @author vegare
 *
 */
public class IncidentWheel extends JButton implements MouseListener {

	private static final long serialVersionUID = 1935827120386000277L;

	private static final int WIDTH = 32;
	private static final int HEIGHT = 32;
	private static final double RADIUS = 13;
	
	private static final double REVOLVE_TIME = 60*60; // seconds
	
	private LogEntrySet model;
	private Collection<LogEntry> logEntries;
	
	/**
	 * Create a new incident wheel
	 * 
	 * @param location Location of the wheel in the map
	 * @param model Underlying model, used to get time base
	 */
	public IncidentWheel(Point location, LogEntrySet model) {
		this(location.x, location.y, model);
	}
	
	/**
	 * Create a new incident wheel
	 * 
	 * @param x Location of the wheel in the map
	 * @param y Location of the wheel in the map
	 * @param model Underlying model, used to get time base
	 */
	public IncidentWheel(int x, int y, LogEntrySet model) {
		
		this.model = model;
		logEntries = new ArrayList<LogEntry>(10);
		
		setOpaque(false);
		
		setLocation(x - WIDTH/2, y - HEIGHT/2);
		setSize(WIDTH, HEIGHT);
		
		addMouseListener(this);
	}
	
	/**
	 * Attach a new incident to the wheel
	 * 
	 * @param entry Incident
	 */
	public void attachEntry(LogEntry entry) {
		logEntries.add(entry);
	}

	/**
	 * Check if a given point is captured by the wheel.
	 * 
	 * @param x X coordinate of point
	 * @param y Y coordinate of point
	 * @return Whether the point is in the wheel
	 */
	public boolean overlaps(int x, int y) {
		return (x >= getX()-WIDTH && x <= getX()+2*WIDTH && y >= getY()-HEIGHT && y <= getY()+2*HEIGHT);
	}

	/**
	 * @return All incidents in the wheel
	 */
	public Collection<LogEntry> getEntries() {
		return logEntries;
	}
	
	/**
	 * @return The most severe level of the wheel incidents
	 */
	private int getMaxPriority() {
		
		int minPri = Integer.MAX_VALUE;
		
		for (LogEntry e : logEntries) {
			if (e.getSeverity() < minPri)
				minPri = e.getSeverity();
		}
		
		return minPri;
	}

	@Override
	public void paint(Graphics g) {
		
		int currentTime = (model.isLive() ? TimeUtils.getCurrentTime()
										  : model.getCurrentTime());
		
		// Draw wheel
		Image img;
		
		switch (getMaxPriority()) {
		case 0: 
		case 1:  img = Resources.getInstance().getImage("wheels/red.png").getImage();
				 break;
		case 2: 
		case 3:  img = Resources.getInstance().getImage("wheels/yellow.png").getImage();
				 break;
		case 4: 
		case 5:  img = Resources.getInstance().getImage("wheels/green.png").getImage();
				 break;
		case 6: 
		case 7: 
		default: img = Resources.getInstance().getImage("wheels/gray.png").getImage();
				 break;
		}
		
		g.drawImage(img, 6, 6, null);
				
		// Draw dots
		g.setColor(Color.BLACK);
		img = Resources.getInstance().getImage("wheels/dot.png").getImage();
		for (LogEntry e : logEntries) {
			
			
			double secondDiff = currentTime - e.getTimestamp();
			double angularMovement = (2.0d*Math.PI*secondDiff / REVOLVE_TIME) + (Math.PI / 2.0d);

			// Find position
			double x = (WIDTH/2)  - RADIUS*Math.cos(angularMovement);
			double y = (HEIGHT/2) - RADIUS*Math.sin(angularMovement);
			
			// Draw
			//g.fillOval((int)(x-3), (int)(y-3), 6, 6);
			
			g.drawImage(img, (int)(x-2), (int)(y-3), null);
		}
		
	}

	@Override
	public void mouseEntered(MouseEvent e) {
		this.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
	}

	@Override
	public void mouseExited(MouseEvent e) {
		this.setCursor(Cursor.getDefaultCursor());
	}

	@Override
	public void mouseClicked(MouseEvent e) {
		ActionEvent evt = new ActionEvent(this, 0, "incidentWheel");
		fireActionPerformed(evt);
	}

	@Override
	public void mousePressed(MouseEvent e) {}

	@Override
	public void mouseReleased(MouseEvent e) {}

}
