package no.ntnu.vegare.visualizer.client.main.overview.map;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionListener;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import javax.swing.JPanel;

import no.ntnu.vegare.visualizer.client.db.LogEntry;
import no.ntnu.vegare.visualizer.client.db.LogEntrySet;
import no.ntnu.vegare.visualizer.client.utils.Resources;

/**
 * <p>The panel containing the world map with incident wheels.</p>
 * 
 * @author vegare
 *
 */
public class MapPanel extends JPanel implements MouseMotionListener, MouseWheelListener {

	private static final long serialVersionUID = 3044786312501228074L;

	private LogEntrySet model;
	
	private Image map;
	
	// Overview of incident wheels currently on display.
	private List<IncidentWheel> incidentWheels;
	
	private Rectangle destRect;
	private ViewPort srcRect;

	private ActionListener listener;
	
	/**
	 * Create a new map panel
	 * 
	 * @param model List of incidents in the view
	 */
	public MapPanel(LogEntrySet model) {
		
		this.model = model;
		
		setLayout(null);
		
		incidentWheels = Collections.synchronizedList(new ArrayList<IncidentWheel>(10));
				
		map = Resources.getInstance().getImage("world-map.jpg").getImage();
		
		destRect = new Rectangle(1000,500);
		srcRect = new ViewPort(map.getWidth(null), map.getHeight(null));
		
		updateWheels();
		
		addMouseListener(srcRect);
		addMouseMotionListener(this);
		addMouseWheelListener(this);
	}
	
	@Override
	protected void paintComponent(Graphics g) {
		super.paintComponent(g);
		
		Graphics2D g2d = (Graphics2D)g;
				
		// Draw background
		g2d.setColor(Color.BLACK);
		g2d.fillRect(0, 0, getWidth(), getHeight());
		
		// Draw map
		g2d.drawImage(map, 	(int)destRect.getX()+2, 	(int)destRect.getY(),
							(int)destRect.getMaxX()-2,	(int)destRect.getMaxY(), 
							(int)srcRect.getX(),		(int)srcRect.getY(),
							(int)srcRect.getMaxX(),		(int)srcRect.getMaxY(),
							null);
	}
	
	/**
	 * <p>Refresh all the wheels. This function:</p>
	 * 
	 * <ul>
	 *  <li>Removes old incidents</li>
	 *  <li>Adds new incidents to wheels</li>
	 * </ul>
	 */
	public synchronized void updateWheels() {
				
		incidentWheels.clear();

		synchronized(model.logEntryLock) {
			for (LogEntry e : model.getEntries()) {
				attachEntry(incidentWheels, e);
			}
		}
				
		this.removeAll();
		
		for (IncidentWheel w : incidentWheels) {
			w.addActionListener(listener);
			this.add(w);
		}
	}
	
	/**
	 * <p>Attach a new security incident to the map, by:</p>
	 * 
	 * <ul>
	 *  <li>Creating a new wheel, or</li>
	 *  <li>attaching incident to an existing wheel</li>
	 * </ul>
	 * 
	 * @param incidentWheels Current incident wheels
	 * @param e New entry
	 */
	private void attachEntry(List<IncidentWheel> incidentWheels, LogEntry e) {
		Point loc = srcRect.locationToPoint(e.getLocation());
		
		if (loc != null) {
			
			boolean attached = false;
			for (IncidentWheel w : incidentWheels) {
				if (w.overlaps(loc.x, loc.y)) {
					w.attachEntry(e);
					attached = true;
				}
			}
			if (!attached) {
				IncidentWheel newWheel = new IncidentWheel(loc, model);
				newWheel.attachEntry(e);
				incidentWheels.add(newWheel);
			}
		}
	}

	public void setActionListener(ActionListener listener) {
		
		this.listener = listener;
	}

	@Override
	public void mouseWheelMoved(MouseWheelEvent e) {
		srcRect.mouseWheelMoved(e);
		updateWheels();
		repaint();
	}


	@Override
	public void mouseDragged(MouseEvent e) {
		srcRect.mouseDragged(e);
		updateWheels();
		repaint();
	}


	@Override
	public void mouseMoved(MouseEvent e) {}
}
