package no.ntnu.vegare.visualizer.client.main.overview.map;

import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;

import com.maxmind.geoip.Location;

/**
 * <p>A model class holding the zoom and pan status of the world map. Receives
 * any mouse input, and modifies the borders and zoom level of the map.</p>
 * 
 * <p>Movement will force an update of incident wheels.</p>
 * 
 * @author vegare
 *
 */
public class ViewPort extends Rectangle implements MouseListener, MouseMotionListener, MouseWheelListener {

	private static final long serialVersionUID = 907979050527488972L;

	private static final Rectangle SOURCE = new Rectangle(0, 0, 1000, 500);
	
	private static final float MIN_SCALE  = 1.0f;
	private static final float MAX_SCALE  = 15.0f;
	private static final float SCALE_STEP = 0.5f;
	
	private float scaleFactor;
	
	private int minX;
	private int minY;
	private int maxX;
	private int maxY;

	private Point pressedPoint;
	private Point pressedLoc;
	
	/**
	 * Create a new viewport
	 * 
	 * @param width Viewport width
	 * @param height Viewport height
	 */
	public ViewPort(int width, int height) {
		super(width, height);
				
		scaleFactor = 1.0f;
		
		minX = minY = 0;
		maxX = width;
		maxY = height;
	}
	
	private Point globalToLocal(int x, int y) {
		Point p = new Point();
		
		p.x = (int) (getScale()*x + getTranslation().x);
		p.y = (int) (getScale()*y + getTranslation().y);
		
		return p;
	}
	
	private Point localToGlobal(int x, int y) {
		Point p = new Point();
		
		p.x = (int) ((x - getTranslation().x) / getScale());
		p.y = (int) ((y - getTranslation().y) / getScale());
		
		return p;
	}
	
	/**
	 * @return Zoom scale of the map
	 */
	private double getScale() {
		return width / SOURCE.getWidth();
	}
	
	/**
	 * @return Current translation
	 */
	private Point getTranslation() {
		return getLocation();
	}
	
	/**
	 * Assure that the user does not pan to non-existing coordinates
	 */
	private void boundCheck() {
		if (x < minX) x = minX;
		if (x + width > maxX) x = maxX - width;
		if (y < minY) y = minY;
		if (y + height > maxY) y = maxY - height;
	}
	
	/**
	 * Convert a geographical location to a point on the map
	 */
	public Point locationToPoint(Location l) {

		if (l == null) return null;
	
		int x = (int) ((maxX/360.0)*l.longitude + (maxX/2));
		int y = (int) (-((maxY)/180.0)*l.latitude + (maxY/2));
		
		return localToGlobal(x, y);
	}
	
	@Override
	public void mouseWheelMoved(MouseWheelEvent e) {

		// Translation parameter
		Point localOld = globalToLocal(e.getX(), e.getY());
		
		// Zoom
		if (e.getWheelRotation() < 0)
			scaleFactor = Math.max(MIN_SCALE, Math.min(MAX_SCALE, scaleFactor+SCALE_STEP));
		else
			scaleFactor = Math.max(MIN_SCALE, Math.min(MAX_SCALE, scaleFactor-SCALE_STEP));
		
		width = (int) ((maxX - minX)/scaleFactor);
		height = (int) ((maxY - minY)/scaleFactor);
		
		// Translate
		Point globalNew = localToGlobal(localOld.x, localOld.y);
		
		x += getScale()*(globalNew.x-e.getX());
		y += getScale()*(globalNew.y-e.getY());
		
		boundCheck();
	}
	
	@Override
	public void mousePressed(MouseEvent e) {
				
		pressedPoint = new Point(e.getX(), e.getY());
		pressedLoc = getLocation();
	}

	@Override
	public void mouseDragged(MouseEvent e) {
		
		Point current = new Point(e.getX(), e.getY());
		
		int diffX = pressedPoint.x - current.x;
		int diffY = pressedPoint.y - current.y;
				
		x = (int) (pressedLoc.x + (diffX*getScale()));
		y = (int) (pressedLoc.y + (diffY*getScale()));
		
		boundCheck();
	}
	
	@Override
	public void mouseMoved(MouseEvent e) {}

	@Override
	public void mouseClicked(MouseEvent e) {}

	@Override
	public void mouseReleased(MouseEvent e) {}

	@Override
	public void mouseEntered(MouseEvent e) {}

	@Override
	public void mouseExited(MouseEvent e) {}
}