package no.ntnu.vegare.visualizer.server.background;

import java.io.IOException;
import java.io.OutputStream;
import java.util.HashMap;
import java.util.Map;

import no.ntnu.vegare.visualizer.server.db.LogDatabase;

import com.sun.net.httpserver.HttpExchange;
import com.sun.net.httpserver.HttpHandler;

/**
 * An implementation of the JAX-WS HTTP handler. Launched in a separate thread
 * when an incoming request occurs, returns JSON incidents or an error message,
 * as specified by the report.
 * 
 * @author vegare
 *
 */
public class RequestHandler implements HttpHandler {

	private LogDatabase db;
	
	public RequestHandler(LogDatabase db) {
		this.db = db;
	}

	@Override
	public void handle(HttpExchange ex) throws IOException {
		
		// Authenticate user using HTTP basic authentication.
		// Hardcoded, as this is just a prototype. Don't use
		// this code for anything besides this prototype!
		if(!ex.getRequestHeaders().containsKey("Authorization")
		|| !"Basic YWRtaW46dGVzdA==".equals(ex.getRequestHeaders().getFirst("Authorization")))
		{
			sendUnauthorized(ex);
		}
		else
		{
			// Check query string
			Map<String, String> query = getQueryMap(ex.getRequestURI().getQuery());
			
			if (query.containsKey("action") && "update".equals(query.get("action"))) {
				sendUpdate(ex, query);
			} else {
				sendVersion(ex);
			}
		}
	}

	/**
	 * Return a server version number
	 * 
	 * @param ex Http exchange instance
	 * @throws IOException If an I/O error occurs.
	 */
	private void sendVersion(HttpExchange ex) throws IOException {
		String response  = "<!DOCTYPE html> "
			+ "<html>"
			+ " <head><title>OK</title></head>"
			+ " <body>LogWheels Prototype</body>"
			+ "</html>";
		
		ex.getResponseHeaders().add("Content-Type", "text/html");
		ex.sendResponseHeaders(200, response.length());
		
		OutputStream os = ex.getResponseBody();
		os.write(response.getBytes());
		os.close();
		
		ex.close();
	}

	/**
	 * Return JSON incidents, as specified by the report.
	 * 
	 * @param ex Http exchange instance
	 * @param query Map of input parameters given in the URL
	 * @throws IOException If an I/O error occurs.
	 */
	private void sendUpdate(HttpExchange ex, Map<String, String> query)
			throws IOException {
		int timeFrom, timeTo, minSig;
		
		try {
			timeFrom = query.containsKey("timeFrom")
					 ? Integer.parseInt(query.get("timeFrom"))
					 : 0;
					 
			timeTo = query.containsKey("timeTo")
				   ? Integer.parseInt(query.get("timeTo"))
				   : 0;		 
		
			minSig = query.containsKey("minSig")
				   ? Integer.parseInt(query.get("minSig"))
				   : 7;	
				   
		} catch (Exception e) {
			sendError(ex);
			return;
		}
				   
		String response = db.getEntries(timeFrom, timeTo, minSig);
		
		ex.getResponseHeaders().add("Content-Type", "application/json");
		ex.sendResponseHeaders(200, response.length());
		
		OutputStream os = ex.getResponseBody();
        os.write(response.getBytes());
	    os.close();
	    ex.close();
	}

	/**
	 * Return an error message, indicating that the user authentication failed.
	 * 
	 * @param ex Http exchange instance
	 * @throws IOException If an I/O error occurs.
	 */
	private void sendUnauthorized(HttpExchange ex) throws IOException {
		String response = "<!DOCTYPE html> "
						+ "<html>"
						+ " <head><title>Error</title></head>"
						+ " <body><h1>401 Unauthorized</h1></body>"
						+ "</html>";
		
		ex.getResponseHeaders().add("WWW-Authenticate", "Basic realm=\"Password Protected Area\"");
		ex.getResponseHeaders().add("Content-Type", "text/html");
		ex.sendResponseHeaders(401, response.length());
		
		OutputStream os = ex.getResponseBody();
		os.write(response.getBytes());
		os.close();
		
		ex.close();
	}

	/**
	 * Send an error message, indicating that an internal error occurred.
	 * 
	 * @param ex Http exchange instance
	 * @throws IOException If an I/O error occurs.
	 */
	private void sendError(HttpExchange ex) throws IOException {
		String response = "<!DOCTYPE html> "
						+ "<html>"
						+ " <head><title>Error</title></head>"
						+ " <body><h1>500 Server Error</h1></body>"
						+ "</html>";

		ex.getResponseHeaders().add("Content-Type", "text/html");
		ex.sendResponseHeaders(500, response.length());
		
		OutputStream os = ex.getResponseBody();
		os.write(response.getBytes());
		os.close();
		
		ex.close();
	}
	
	/**
	 * Extract a map of query parameters form the incoming URL.
	 * 
	 * @param query URL string
	 * @return Map of query parameters
	 */
	private static Map<String, String> getQueryMap(String query)  
	{
		if (query == null) return new HashMap<String, String>();
		
		String[] params = query.split("&");
		Map<String, String> map = new HashMap<String, String>();  		
		for (String param : params)  
		{  
			String name = param.split("=")[0].trim();  
			String value = param.split("=")[1].trim();  
			map.put(name, value);  
	    }  
	    return map;  
	}
}
