package no.ntnu.vegare.visualizer.server.background;

import java.io.IOException;
import java.lang.Thread.UncaughtExceptionHandler;
import java.net.InetSocketAddress;

import com.sun.net.httpserver.HttpServer;

import no.ntnu.vegare.visualizer.server.db.LogDatabase;

/**
 * <p>Thread safe singleton implementation by Bill Pugh:
 * <a href="http://en.wikipedia.org/wiki/Singleton_pattern">
 * http://en.wikipedia.org/wiki/Singleton_pattern</a></p>
 * 
 * <p>Holds all background threads in the application, and verifies that these 
 * are not interfering with each other.</p>
 * 
 * @author vegare
 *
 */
public class ThreadPool {

	private GeneratorThread generatorThread;
	private HttpServer server;
	
	// Private constructor prevents instantiation from other classes
	private ThreadPool() {
	
	}
	 
	/**
	 * SingletonHolder is loaded on the first execution of Singleton.getInstance() 
	 * or the first access to SingletonHolder.INSTANCE, not before.
	 */
	private static class SingletonHolder { 
		public static final ThreadPool INSTANCE = new ThreadPool();
	}
	 
	public static ThreadPool getInstance() {
		return SingletonHolder.INSTANCE;
	}
	
	/**
	 * Start a new HTTP server on a specified port.
	 * 
	 * @param db Database containing target incidents
	 * @param port TCP port to run HTTP server on
	 * @param eh An UncaughtExceptionHandler which receives unexpected errors.
	 * @return "OK" if server was started, error msg if an exception occurred.
	 */
	public String startServer(LogDatabase db, int port, UncaughtExceptionHandler eh) {
		
		try {
			server = HttpServer.create(new InetSocketAddress(port), 0);
			server.createContext("/", new RequestHandler(db));
			server.start();
			
		} catch (IOException e) {
			return "Could not start server on port " + port;
		}
		
		return "OK";
	}
	
	/**
	 * Stop the server from executing.
	 * 
	 * @return Always "OK" in the prototype.
	 */
	public String stopServer() {
		
		server.stop(0);
		
		return "OK";
	}
	
	/**
	 * Start a generator thread, creating random incidents.
	 * 
	 * @param db Database to load incidents into
	 * @param perHour Average incidents per hour
	 * @return Always "OK" in the prototype.
	 */
	public String startGenerator(LogDatabase db, int perHour) {
		
		if (generatorThread != null
		&&  generatorThread.isRunning())
			return "Generator is already running";
				
		generatorThread = new GeneratorThread(db, perHour);
		new Thread(generatorThread).start();
		
		return "OK";
	}
	
	/**
	 * Stop the generator thread from executing.
	 * 
	 * @return "OK" on success, otherwise error message
	 */
	public String stopGenerator() {
		
		if (generatorThread == null)
			return "Generator is not running";
		
		generatorThread.stop();
		return "OK";
	}
}
