package no.ntnu.vegare.visualizer.server.db;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import no.ntnu.vegare.visualizer.server.utils.DataGenerator;

/**
 * <p>A simplified implementation of a SQLite database interface.</p>
 * 
 * <p>Uses SqliteJDBC (<a href="http://www.zentus.com/sqlitejdbc/">
 * http://www.zentus.com/sqlitejdbc/</a>) to create an in-memory database, and
 * provides an interface to insert or load entries.</p>
 * 
 * <p>The database is cleared when the process dies.</p>
 * 
 * @author vegare
 *
 */
public class LogDatabase {

	private Connection conn;

	/**
	 * Create an in-memory database and a table for log records.
	 */
	public LogDatabase() {

		try {
			Class.forName("org.sqlite.JDBC");
		
		    conn = DriverManager.getConnection("jdbc:sqlite::memory:");
		    
		    Statement stat = conn.createStatement();
		    stat.executeUpdate("drop table if exists serverlog;");
		    stat.executeUpdate("create table serverlog (id, timestamp, severity, facility, ipaddress, message);");
		      
		} catch (Exception e) {
			// Oh, the horror... Just a prototype!
			e.printStackTrace();
		}
	}

	/**
	 * <p>Add a new security incident to the database.</p>
	 * 
	 * <p>Severity is given as one of the following:</p>
	 * 
	 * <ul>
	 *  <li>0 - Emergency: system is unusable</li>
	 *  <li>1 - Alert: action must be taken immediately</li>
	 *  <li>2 - Critical: critical conditions</li>
	 *  <li>3 - Error: error conditions</li>
	 *  <li>4 - Warning: warning conditions</li>
	 *  <li>5 - Notice: normal but significant condition</li>
	 *  <li>6 - Informational: informational messages</li>
	 *  <li>7 - Debug: debug-level messages</li>
	 * </ul>
	 * 
	 * @param severity Integer value from 0-7
	 * @param facility Textual name of target system
	 * @param ip IP addres as a dot-decimal string
	 * @param msg Textual message
	 */
	public synchronized void addEntry(int severity, String facility,
									  String ip, String msg) {

		try {
		    PreparedStatement prep = conn.prepareStatement(
		      "insert into serverlog values (?, ?, ?, ?, ?, ?);");
	
		    prep.setInt(1, DataGenerator.getId());
		    prep.setInt(2, DataGenerator.getTimestamp());
		    prep.setInt(3, severity);
		    prep.setString(4, facility);
		    prep.setString(5, ip);
		    prep.setString(6, msg);
		    
		    prep.addBatch();
	
		    conn.setAutoCommit(false);
		    prep.executeBatch();
		    conn.setAutoCommit(true);
		      
		} catch (Exception ex) {
			// Oh, the horror... Just a prototype!
			ex.printStackTrace();
		}
	}
	
	/**
	 * <p>Load all incidents occurring in a time interval, specified by two
	 * UNIX timestamps. In addition, filtering can be performed on severity,
	 * avoiding routine messages.</p>
	 * 
	 * @param startTime Earliest time, as UNIX timestamp
	 * @param stopTime Latest time, as UNIX timestamp
	 * @param minSev Minimum severity to load
	 * @return JSON formatted string of incidents
	 */
	public synchronized String getEntries(int startTime, int stopTime, int minSev) {
		
		try {
			
			String sql = "select * from serverlog where " + 
						 "timestamp >= ? and " +
						 "timestamp <= ? and " +
						 "severity <= ?";
						
			PreparedStatement prep = conn.prepareStatement(sql);
			
		    prep.setInt(1, startTime);
		    prep.setInt(2, stopTime);
		    prep.setInt(3, minSev);
		    
		    ResultSet rs = prep.executeQuery();
		    
		    StringBuilder JSON = new StringBuilder();
		    
		    while (rs.next()) {

			    addEntry(JSON, rs);
		    }
			
		    return JSON.toString();
			
		} catch (SQLException e) {
			e.printStackTrace();
		}
		
		return "OK";
	}

	/**
	 * Convert a result set to a JSON string.
	 * 
	 * @param JSON The string builder to load with data
	 * @param rs The result set from the SqliteJDBC instance
	 * @throws SQLException If the database scheme does not correspond
	 * to the method. Should not happen unless source code is changed.
	 */
	private void addEntry(StringBuilder JSON, ResultSet rs) throws SQLException {

		JSON.append("{");
		JSON.append("\"id\": ");
		JSON.append(rs.getInt("id"));
		JSON.append(", \"timestamp\": ");
		JSON.append(rs.getInt("timestamp"));
		JSON.append(", \"severity\": ");
		JSON.append(rs.getInt("severity"));
		JSON.append(", \"facility\": \"");
		JSON.append(rs.getString("facility"));
		JSON.append("\", \"ipaddress\": \"");
		JSON.append(rs.getString("ipaddress"));
		JSON.append("\", \"message\": \"");
		JSON.append(rs.getString("message"));		
		JSON.append("\"}\r\n");
	}
}
