package no.ntnu.vegare.visualizer.server.gui.panels;

import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.lang.Thread.UncaughtExceptionHandler;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;

import no.ntnu.vegare.visualizer.server.background.ThreadPool;
import no.ntnu.vegare.visualizer.server.db.LogDatabase;
import no.ntnu.vegare.visualizer.server.utils.Resources;

/**
 * <p>Panel controlling the HTTP server listening to client requests.</p>
 * 
 * <p>Uses the HTTP library JAX-WS available from
 * <a href="http://jax-ws.java.net/">http://jax-ws.java.net/</a></p>
 * 
 * <p>Serves both as a view and a controller, for simplicity reasons. The
 * method <code>actionPerformed</code> handles clicks on the two buttons,
 * and starts or stops the server through the ThreadPool class.</p>
 * 
 * <p>This class also accepts uncaught exceptions from the RequestHandler
 * thread, and displays an error message if any occur.</p>
 * 
 * @see ThreadPool
 * @see RequestHandler
 * @author vegare
 *
 */
public class ServerPanel extends JPanel implements ActionListener, UncaughtExceptionHandler {

	private static final long serialVersionUID = 7597046945327863164L;

	private LogDatabase db;
	
	private JLabel serverLabel;
	private JSpinner portNumber;
	private JButton startServer;
	private JButton stopServer;
	
	public ServerPanel(LogDatabase db) {

		this.db = db;
		setLayout(new FlowLayout(FlowLayout.CENTER, 7, 10));
		initializeComponents();
	}
	
	/**
	 * Initialize the panels with positioning and layout.
	 * A simple flowlayout is used.
	 */
	private void initializeComponents() {
		
		serverLabel = new JLabel("Start server on port");
		
		portNumber = new JSpinner(new SpinnerNumberModel(8080, 0, 65535, 1));
		portNumber.setEditor(new JSpinner.NumberEditor(portNumber, "####"));
		
		startServer = new JButton("Start", Resources.getInstance().getImage("start.png"));
		startServer.setPreferredSize(new Dimension(100, 30));
		startServer.addActionListener(this);
		
		stopServer = new JButton("Stop", Resources.getInstance().getImage("stop.png"));
		stopServer.setPreferredSize(new Dimension(100, 30));
		stopServer.setEnabled(false);
		stopServer.addActionListener(this);
		
		add(serverLabel);
		add(portNumber);
		add(startServer);
		add(stopServer);
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		
		if (e.getSource() == startServer) {
			
			String msg = ThreadPool.getInstance().startServer(db, (Integer) portNumber.getValue(), this);
			
			if("OK".equals(msg)) {
				startServer.setEnabled(false);
				stopServer.setEnabled(true);
			} else {
				JOptionPane.showMessageDialog(this, msg);
			}
			
		} else if (e.getSource() == stopServer) {
			
			String msg = ThreadPool.getInstance().stopServer();
			
			if("OK".equals(msg)) {
				startServer.setEnabled(true);
				stopServer.setEnabled(false);
			} else {
				JOptionPane.showMessageDialog(this, msg);
			}
			
		}
	}

	@Override
	public void uncaughtException(Thread t, Throwable e) {
		
		if ("server".equals(t.getName())) {
			
			JOptionPane.showMessageDialog(this, e.getMessage());
			startServer.setEnabled(true);
			stopServer.setEnabled(false);
		}
	}
}
